//******************************************************************************
//
// File:    ControllableChannel.java
// Package: edu.rit.m2mp
// Unit:    Class edu.rit.m2mp.ControllableChannel
//
// This Java source file is copyright (C) 2006 by Alan Kaminsky. All rights
// reserved. For further information, contact the author, Alan Kaminsky, at
// ark@cs.rit.edu.
//
// This Java source file is part of the M2MI Library ("The Library"). The
// Library is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// The Library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// A copy of the GNU General Public License is provided in the file gpl.txt. You
// may also obtain a copy of the GNU General Public License on the World Wide
// Web at http://www.gnu.org/licenses/gpl.html or by writing to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//******************************************************************************

package edu.rit.m2mp;

import java.io.IOException;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JRootPane;

/**
 * Class ControllableChannel provides a user-controllable channel in the
 * Many-to-Many Protocol (M2MP). The M2MP Layer uses a channel to send messages
 * to and receive messages from the external network. The
 * <TT>edu.rit.m2mp.channel.class</TT> property in the M2MP properties file
 * specifies the type of channel the M2MP Layer uses to communicate with the
 * external network. If the <TT>edu.rit.m2mp.debug.controlpanel</TT> property is
 * set to 1, the M2MP Layer interposes a ControllableChannel between itself and
 * the external channel. (See class {@linkplain M2MPProperties} for further
 * information about the M2MP properties file.)
 * <P>
 * The ControllableChannel, if present, displays this UI:
 * <CENTER>
 * <IMG SRC="doc-files/ControllableChannelUI.png">
 * </CENTER>
 * <P>
 * The "Network" setting specifies whether the M2MP Layer is connected to the
 * external network. If the "Connected" button is selected, the
 * ControllableChannel passes all incoming and outgoing packets straight
 * through; the M2MP Layer is connected to the external network. If the
 * "Disconnected" button is selected, the ControllableChannel drops all incoming
 * and outgoing packets; the M2MP Layer is disconnected from the external
 * network. This can be used to simulate a network failure or network partition.
 *
 * @author  Alan Kaminsky
 * @version 29-Mar-2006
 */
public class ControllableChannel
	extends Channel
	{

// Hidden constants.

	private static final int GAP = 10;

// Hidden data members.

	private Channel myChannel;

	private UI myUI;
	private boolean myNetworkConnected = true;

// Hidden helper classes.

	/**
	 * Class UI provides the user interface for the ControllableChannel.
	 *
	 * @author  Alan Kaminsky
	 * @version 28-Mar-2006
	 */
	private class UI
		extends JFrame
		{
		public UI()
			{
			super ("M2MP Control Panel");

			JPanel panela = new JPanel();
			getContentPane().add (panela);
			panela.setBorder
				(BorderFactory.createEmptyBorder (GAP, GAP, GAP, GAP));
			panela.setLayout (new BoxLayout (panela, BoxLayout.X_AXIS));

			panela.add (new JLabel ("Network:"));

			panela.add (Box.createHorizontalStrut (GAP));

			ButtonGroup networkButtons = new ButtonGroup();

			JRadioButton connectedButton = new JRadioButton ("Connected");
			panela.add (connectedButton);
			networkButtons.add (connectedButton);
			connectedButton.addActionListener (new ActionListener()
				{
				public void actionPerformed
					(ActionEvent event)
					{
					myNetworkConnected = true;
					}
				});

			panela.add (Box.createHorizontalStrut (GAP));

			JRadioButton disconnectedButton = new JRadioButton ("Disconnected");
			panela.add (disconnectedButton);
			networkButtons.add (disconnectedButton);
			disconnectedButton.addActionListener (new ActionListener()
				{
				public void actionPerformed
					(ActionEvent event)
					{
					myNetworkConnected = false;
					}
				});

			networkButtons.setSelected (connectedButton.getModel(), true);

			setDefaultCloseOperation (JFrame.DO_NOTHING_ON_CLOSE);
			pack();
			}
		}

// Exported constructors.

	/**
	 * Construct a new controllable channel. To receive incoming packets, the
	 * controllable channel may obtain {@link Packet </CODE>Packet<CODE>}
	 * objects from the given {@link PacketPool </CODE>PacketPool<CODE>}. The
	 * controllable channels will pass packets through to and from the given
	 * channel.
	 *
	 * @param  thePacketPool  Packet pool.
	 * @param  theChannel     Underlying channel.
	 *
	 * @exception  NullPointerException
	 *     (unchecked exception) Thrown if <TT>thePacketPool</TT> is null or
	 *     <TT>theChannel</TT> is null.
	 */
	public ControllableChannel
		(PacketPool thePacketPool,
		 Channel theChannel)
		{
		super (thePacketPool);
		if (theChannel == null)
			{
			throw new NullPointerException
				("ControllableChannel(): theChannel is null");
			}
		myChannel = theChannel;
		myUI = new UI();
		myUI.setVisible (true);
		}

// Exported operations.

	/**
	 * Receive an M2MP packet via this channel.
	 *
	 * @return  M2MP packet that was received.
	 *
	 * @exception  IOException
	 *     Thrown if an I/O error occurred.
	 */
	public Packet receivePacket()
		throws IOException
		{
		Packet packet;
		do
			{
			packet = myChannel.receivePacket();
			}
		while (! myNetworkConnected);
		return packet;
		}

	/**
	 * Send the given M2MP packet via this channel.
	 *
	 * @param  thePacket  M2MP packet to be sent.
	 *
	 * @exception  IOException
	 *     Thrown if an I/O error occurred.
	 */
	public void transmitPacket
		(Packet thePacket)
		throws IOException
		{
		if (myNetworkConnected) myChannel.transmitPacket (thePacket);
		}

	}
