//******************************************************************************
//
// File:    SurveyState.java
// Package: edu.rit.survey
// Unit:    Class edu.rit.survey.SurveyState
//
// This Java source file is copyright (C) 2006 by Alan Kaminsky. All rights
// reserved. For further information, contact the author, Alan Kaminsky, at
// ark@cs.rit.edu.
//
// This Java source file is part of the M2MI Library ("The Library"). The
// Library is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// The Library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// A copy of the GNU General Public License is provided in the file gpl.txt. You
// may also obtain a copy of the GNU General Public License on the World Wide
// Web at http://www.gnu.org/licenses/gpl.html or by writing to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//******************************************************************************

package edu.rit.survey;

import edu.rit.m2mi.Eoid;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import java.util.HashMap;
import java.util.Map;

/**
 * Class SurveyState encapsulates the state of a survey. The survey state
 * consists of the name; the question; four answers; and the number of votes for
 * each answer.
 * <P>
 * The survey state's name, question, and answers cannot be changed once
 * constructed. Survey state objects are compared for equality based on the
 * name, question, and answers.
 * <P>
 * To deal with the lack of a central server in an ad hoc collaborative
 * application, the survey state object keeps track of each participant's vote
 * individually using an instance of class {@linkplain SurveyVote}. Each
 * participant is identified by a unique {@linkplain edu.rit.m2mi.Eoid Eoid},
 * which maps to the participant's vote. A timestamp (logical clock value) is
 * associated with each vote as well. The <TT>merge()</TT> operation combines
 * two survey state objects, retaining the highest-timestamped vote for each
 * participant.
 *
 * @author  Alan Kaminsky
 * @version 25-Mar-2006
 */
public class SurveyState
	implements Externalizable
	{

// Exported constants.

	/**
	 * The number of answers to a survey question (4).
	 */
	public static final int NUM_ANSWERS = 4;

// Hidden data members.

	private static final long serialVersionUID = 5119785749523181697L;

	// Immutable characterestics and hash code thereof.
	String myName;
	String myQuestion;
	String[] myAnswer = new String [NUM_ANSWERS];
	int myHashCode;

	// Mapping from participant ID to participant's vote.
	HashMap<Eoid,SurveyVote> myVoteMap = new HashMap<Eoid,SurveyVote>();

	// Total vote count for each answer, computed from the map's contents.
	int[] myVoteCount;

// Exported constructors.

	/**
	 * Construct a new empty survey state object. This constructor is for use
	 * only during object deserialization.
	 */
	public SurveyState()
		{
		}

	/**
	 * Construct a new survey state object. Initially, the name, question, and
	 * answers are given by the arguments and the vote counts are 0.
	 *
	 * @param  theName      Survey name.
	 * @param  theQuestion  Survey question.
	 * @param  theAnswer    Array of survey answers.
	 *
	 * @exception  NullPointerException
	 *     (unchecked exception) Thrown if <TT>theName</TT>,
	 *     <TT>theQuestion</TT>, <TT>theAnswer</TT>, or any element of
	 *     <TT>theAnswer</TT> is null.
	 * @exception  IllegalArgumentException
	 *     (unchecked exception) Thrown if <TT>theAnswer</TT> does not have the
	 *     correct number of elements.
	 */
	public SurveyState
		(String theName,
		 String theQuestion,
		 String[] theAnswer)
		{
		if (theName == null)
			{
			throw new NullPointerException
				("SurveyState(): theName is null");
			}
		if (theQuestion == null)
			{
			throw new NullPointerException
				("SurveyState(): theQuestion is null");
			}
		if (theAnswer == null)
			{
			throw new NullPointerException
				("SurveyState(): theAnswer is null");
			}
		if (theAnswer.length != NUM_ANSWERS)
			{
			throw new NullPointerException
				("SurveyState(): theAnswer does not have " + NUM_ANSWERS +
				 " elements");
			}
		for (int i = 0; i < NUM_ANSWERS; ++ i)
			{
			if (theAnswer[i] == null)
				{
				throw new NullPointerException
					("SurveyState(): theAnswer[" + i + "] is null");
				}
			}

		myName = theName;
		myQuestion = theQuestion;
		myAnswer = theAnswer;
		computeHashCode();
		}

// Exported operations.

	/**
	 * Get this survey state's name.
	 *
	 * @return  Name.
	 */
	public String getName()
		{
		return myName;
		}

	/**
	 * Get this survey state's question.
	 *
	 * @return  Question.
	 */
	public String getQuestion()
		{
		return myQuestion;
		}

	/**
	 * Get one of this survey state's answers.
	 *
	 * @param  i  Answer index (0-3).
	 *
	 * @return  Answer.
	 */
	public String getAnswer
		(int i)
		{
		return myAnswer[i];
		}

	/**
	 * Get one of this survey state's vote counts.
	 *
	 * @param  i  Answer index (0-3).
	 *
	 * @return  Vote count.
	 */
	public int getVoteCount
		(int i)
		{
		// Recalculate vote counts if necessary.
		if (myVoteCount == null)
			{
			myVoteCount = new int [NUM_ANSWERS];
			for (SurveyVote vote : myVoteMap.values())
				{
				if (vote.myTimestamp > 0)
					{
					++ myVoteCount[vote.myVote];
					}
				}
			}
		return myVoteCount[i];
		}

	/**
	 * Get the survey vote object for the given participant ID in this survey
	 * state object.
	 *
	 * @param  id  Participant ID.
	 *
	 * @return  Survey vote object, or null if there is none.
	 */
	public SurveyVote getSurveyVote
		(Eoid id)
		{
		return myVoteMap.get (id);
		}

	/**
	 * Set the survey vote object for the given participant ID in this survey
	 * state object.
	 *
	 * @param  id             Participant ID.
	 * @param  theSurveyVote  Survey vote object.
	 *
	 * @exception  NullPointerException
	 *     (unchecked exception) Thrown if <TT>theSurveyVote</TT> is null.
	 */
	public void setSurveyVote
		(Eoid id,
		 SurveyVote theSurveyVote)
		{
		if (theSurveyVote == null)
			{
			throw new NullPointerException
				("SurveyState.setSurveyVote(): theSurveyVote is null");
			}
		myVoteMap.put (id, theSurveyVote);
		recalculate();
		}

	/**
	 * Recalculate the vote counts in this survey state object.
	 */
	public void recalculate()
		{
		myVoteCount = null;
		}

	/**
	 * Merge the given survey state object into this survey state object.
	 * Afterwards, this survey state object reflects the most recent vote for
	 * each participant.
	 *
	 * @param  theSurveyState  Survey state object to merge.
	 */
	public void merge
		(SurveyState theSurveyState)
		{
		// Process each entry in theSurveyState's vote map.
		for (Map.Entry<Eoid,SurveyVote> entry :
					theSurveyState.myVoteMap.entrySet())
			{
			// Get participant ID and vote from theSurveyState.
			Eoid id = entry.getKey();
			SurveyVote thatVote = entry.getValue();

			// Get vote from this survey state.
			SurveyVote thisVote = this.myVoteMap.get (id);

			// If participant ID does not exist in this survey state, or
			// theSurveyState's timestamp is more recent, replace vote in this
			// survey state.
			if (thisVote == null || thatVote.myTimestamp > thisVote.myTimestamp)
				{
				this.myVoteMap.put (id, thatVote);
				this.recalculate();
				}
			}
		}

	/**
	 * Write this survey state to the given object output stream.
	 *
	 * @param  out  Object output stream.
	 *
	 * @exception  IOException
	 *     Thrown if an I/O error occurred.
	 */
	public void writeExternal
		(ObjectOutput out)
		throws IOException
		{
		out.writeUTF (myName);
		out.writeUTF (myQuestion);
		for (int i = 0; i < NUM_ANSWERS; ++ i)
			{
			out.writeUTF (myAnswer[i]);
			}
		out.writeInt (myVoteMap.size());
		for (Map.Entry<Eoid,SurveyVote> entry : myVoteMap.entrySet())
			{
			entry.getKey().write (out);
			entry.getValue().write (out);
			}
		}

	/**
	 * Read this survey state from the given object input stream.
	 *
	 * @param  in  Object input stream.
	 *
	 * @exception  IOException
	 *     Thrown if an I/O error occurred.
	 */
	public void readExternal
		(ObjectInput in)
		throws IOException
		{
		myName = in.readUTF();
		myQuestion = in.readUTF();
		for (int i = 0; i < NUM_ANSWERS; ++ i)
			{
			myAnswer[i] = in.readUTF();
			}
		computeHashCode();
		int n = in.readInt();
		for (int i = 0; i < n; ++ i)
			{
			Eoid eoid = new Eoid();
			SurveyVote vote = new SurveyVote();
			eoid.read (in);
			vote.read (in);
			myVoteMap.put (eoid, vote);
			}
		}

	/**
	 * Determine if this survey state is equal to the given object.
	 *
	 * @param  obj  Object to test.
	 *
	 * @return  True if this survey state is equal to <TT>obj</TT>, false
	 *          otherwise.
	 */
	public boolean equals
		(Object obj)
		{
		if (obj instanceof SurveyState)
			{
			SurveyState that = (SurveyState) obj;
			if (! this.myName.equals (that.myName)) return false;
			if (! this.myQuestion.equals (that.myQuestion)) return false;
			for (int i = 0; i < NUM_ANSWERS; ++ i)
				{
				if (! this.myAnswer[i].equals (that.myAnswer[i])) return false;
				}
			return true;
			}
		return false;
		}

	/**
	 * Returns a hash code for this survey state.
	 *
	 * @return  Hash code.
	 */
	public int hashCode()
		{
		return myHashCode;
		}

	/**
	 * Returns a string version of this survey state. The string is just the
	 * name of this survey state.
	 *
	 * @return  String version.
	 */
	public String toString()
		{
		return myName;
		}

// Hidden operations.

	/**
	 * Compute the hash code for this survey state.
	 */
	private void computeHashCode()
		{
		myHashCode = myName.hashCode();
		myHashCode += myQuestion.hashCode();
		for (int i = 0; i < NUM_ANSWERS; ++ i)
			{
			myHashCode += myAnswer[i].hashCode();
			}
		}

	/**
	 * Dump this survey state to the standard output.
	 *
	 * @param  msg  Message.
	 */
	private void dump
		(String msg)
		{
		System.out.println (msg);
		for (Map.Entry<Eoid,SurveyVote> entry : myVoteMap.entrySet())
			{
			System.out.print (entry.getKey());
			System.out.print ('\t');
			System.out.print (entry.getValue().myTimestamp);
			System.out.print ('\t');
			System.out.print (entry.getValue().myVote);
			System.out.println();
			}
		}

	}
